from .baserun import Baserun
from .unittests_class import BaserunTestCase

init = Baserun.init
test = Baserun.test
log = Baserun.log
log_llm_chat = Baserun.log_llm_chat
log_llm_completion = Baserun.log_llm_completion
flush = Baserun.flush



import unittest
import baserun
from unittest.mock import MagicMock, patch


# Run using BASERUN_API_KEY="test-key" python3 -m unittest tests/unittests_test.py
class BaserunTests(baserun.BaserunTestCase):

    def setUp(self):
        self.mock_baserun_store = MagicMock()
        patcher = patch('baserun.Baserun.store_test', new=self.mock_baserun_store)
        self.addCleanup(patcher.stop)
        patcher.start()

        self.next_assert = None

    def tearDown(self):
        if self.next_assert:
            self.next_assert()

    def test_log(self):
        log_name = "TestEvent"
        baserun.log(log_name, 'whatever')

        def assertion_func():
            stored_data = self.mock_baserun_store.call_args[0][0]
            self.assertIn(log_name, stored_data['steps'][0]['name'])
            self.assertIn('whatever', stored_data['steps'][0]['payload'])

        self.next_assert = assertion_func

    def test_log_with_payload(self):
        event_name = "TestEvent"
        event_payload = {
            "action": "called_api",
            "value": 42
        }

        baserun.log(event_name, payload=event_payload)

        def assertion_func():
            stored_data = self.mock_baserun_store.call_args[0][0]
            self.assertIn(event_name, stored_data['steps'][0]['name'])
            self.assertEqual(event_payload, stored_data['steps'][0]['payload'])

        self.next_assert = assertion_func

    def test_multiple_logs_same_baserun_id(self):
        log_name_1 = "TestEvent1"
        log_name_2 = "TestEvent2"
        baserun.log(log_name_1, 'string payload 1')
        baserun.log(log_name_2, 'string payload 2')

        def assertion_func():
            stored_data = self.mock_baserun_store.call_args[0][0]
            self.assertIn(log_name_1, stored_data['steps'][0]['name'])
            self.assertIn(log_name_2, stored_data['steps'][1]['name'])
            self.assertIn('string payload 1', stored_data['steps'][0]['payload'])
            self.assertIn('string payload 2', stored_data['steps'][1]['payload'])

        self.next_assert = assertion_func

    def test_log_llm_chat(self):
        config = {
            "model": "gpt-3.5-turbo",
            "temperature": 0.7
        }
        messages = [
            {"role": "system", "content": "You are a helpful assistant."},
            {"role": "user", "content": "Who won the world series in 2020?"}
        ]
        output = "The Los Angeles Dodgers won the World Series in 2020."

        baserun.log_llm_chat(config=config, messages=messages, output=output)

        def assertion_func():
            stored_data = self.mock_baserun_store.call_args[0][0]
            self.assertEqual(stored_data['steps'][0]['type'], "chat")
            self.assertEqual(stored_data['steps'][0]['provider'], "openai")
            self.assertEqual(stored_data['steps'][0]['config']['model'], "gpt-3.5-turbo")
            self.assertEqual(stored_data['steps'][0]['messages'], [
                {"role": "system", "content": "You are a helpful assistant."},
                {"role": "user", "content": "Who won the world series in 2020?"}
            ])
            self.assertEqual(stored_data['steps'][0]['output'], "The Los Angeles Dodgers won the World Series in 2020.")

        self.next_assert = assertion_func

    def test_log_llm_completion(self):
        config = {
            "model": "text-davinci-003",
            "temperature": 0.8,
            "max_tokens": 100
        }
        prompt = "Once upon a time, there was a {character} who {action}."
        variables = {"character": "brave knight", "action": "fought dragons"}

        baserun.log_llm_completion(config=config, prompt=prompt, output="Some random output", variables=variables)

        def assertion_func():
            stored_data = self.mock_baserun_store.call_args[0][0]
            self.assertEqual(stored_data['steps'][0]['type'], "completion")
            self.assertEqual(stored_data['steps'][0]['provider'], "openai")
            self.assertEqual(stored_data['steps'][0]['config']['model'], "text-davinci-003")
            self.assertEqual(stored_data['steps'][0]['prompt'], "Once upon a time, there was a {{character}} who {{action}}.")
            self.assertEqual(stored_data['steps'][0]['variables'], {"character": "brave knight", "action": "fought dragons"})
            self.assertEqual(stored_data['steps'][0]['output'], "Some random output")

        self.next_assert = assertion_func


if __name__ == '__main__':
    unittest.main()



import unittest
from unittest.mock import patch
import baserun
import os


# Run using python3 -m unittest tests/explicit_init.py
class TestBaserunInit(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        os.environ["BASERUN_API_KEY"] = "test-key"
        cls.api_key = "test_api_key"
        cls.api_url = "https://baserun.ai/api/v1/runs"
        baserun.init(cls.api_url)

    @patch("baserun.flush")
    @patch("baserun.Baserun.store_test")
    def test_explicit_log(self, mock_store_test, mock_flush):

        @baserun.test
        def sample_test():
            baserun.log("TestEvent", "string payload")

        sample_test()
        baserun.flush()

        mock_store_test.assert_called_once()
        stored_data = mock_store_test.call_args[0][0]
        self.assertEqual(stored_data['steps'][0]['name'], "TestEvent")
        self.assertEqual(stored_data['steps'][0]['payload'], "string payload")

        mock_flush.assert_called_once()

    @patch("baserun.flush")
    @patch("baserun.Baserun.store_test")
    def test_explicit_log_with_payload(self, mock_store_test, mock_flush):
        log_name = "TestEvent"
        log_payload = {
            "action": "called_api",
            "value": 42
        }

        @baserun.test
        def sample_test():
            baserun.log(log_name, log_payload)

        sample_test()
        baserun.flush()

        mock_store_test.assert_called_once()
        stored_data = mock_store_test.call_args[0][0]
        self.assertEqual(stored_data['steps'][0]['name'], log_name)
        self.assertEqual(stored_data['steps'][0]['payload'], log_payload)

        mock_flush.assert_called_once()

    @patch("baserun.flush")
    @patch("baserun.Baserun.store_test")
    def test_log_llm_chat(self, mock_store_test, mock_flush):
        @baserun.test
        def llm_chat_test():
            config = {
                "model": "gpt-3.5-turbo",
                "temperature": 0.7
            }
            messages = [
                {"role": "system", "content": "You are a helpful assistant."},
                {"role": "user", "content": "Who won the world series in 2020?"}
            ]
            output = "The Los Angeles Dodgers won the World Series in 2020."

            baserun.log_llm_chat(config=config, messages=messages, output=output)

        llm_chat_test()
        baserun.flush()

        mock_store_test.assert_called_once()
        stored_data = mock_store_test.call_args[0][0]
        self.assertEqual(stored_data['steps'][0]['type'], "chat")
        self.assertEqual(stored_data['steps'][0]['provider'], "openai")
        self.assertEqual(stored_data['steps'][0]['config']['model'], "gpt-3.5-turbo")
        self.assertEqual(stored_data['steps'][0]['messages'], [
            {"role": "system", "content": "You are a helpful assistant."},
            {"role": "user", "content": "Who won the world series in 2020?"}
        ])
        self.assertEqual(stored_data['steps'][0]['output'], "The Los Angeles Dodgers won the World Series in 2020.")

        mock_flush.assert_called_once()

    @patch("baserun.flush")
    @patch("baserun.Baserun.store_test")
    def test_log_llm_completion(self, mock_store_test, mock_flush):
        @baserun.test
        def llm_completion_test():
            config = {
                "model": "text-davinci-003",
                "temperature": 0.8,
                "max_tokens": 100
            }
            prompt = "Once upon a time, there was a {character} who {action}."

            baserun.log_llm_completion(config=config, prompt=prompt, output="Some random output", variables={"character": "brave knight", "action": "fought dragons"})

        llm_completion_test()
        baserun.flush()

        mock_store_test.assert_called_once()
        stored_data = mock_store_test.call_args[0][0]
        self.assertEqual(stored_data['steps'][0]['type'], "completion")
        self.assertEqual(stored_data['steps'][0]['provider'], "openai")
        self.assertEqual(stored_data['steps'][0]['config']['model'], "text-davinci-003")
        self.assertEqual(stored_data['steps'][0]['prompt'], "Once upon a time, there was a {{character}} who {{action}}.")
        self.assertEqual(stored_data['steps'][0]['variables'], {"character": "brave knight", "action": "fought dragons"})
        self.assertEqual(stored_data['steps'][0]['output'], "Some random output")

        mock_flush.assert_called_once()


if __name__ == '__main__':
    unittest.main()



import baserun
from baserun import baserun as baserun_module


# Run using BASERUN_API_KEY="test-key" pytest --baserun --no-flush tests/pytest_test.py
def test_log_message():
    log_name = "TestEvent"
    baserun.log(log_name, "whatever")

    stored_data = baserun_module._thread_local.buffer
    assert stored_data[0]['stepType'] == 'log'
    assert log_name in stored_data[0]['name']
    assert "whatever" in stored_data[0]['payload']


def test_multiple_logs_same_baserun_id():
    log_name_1 = "TestEvent1"
    log_name_2 = "TestEvent2"

    baserun.log(log_name_1, 'string payload 1')
    baserun.log(log_name_2, 'string payload 2')

    stored_data = baserun_module._thread_local.buffer
    assert stored_data[0]['stepType'] == 'log'
    assert stored_data[1]['stepType'] == 'log'
    assert log_name_1 in stored_data[0]['name']
    assert log_name_2 in stored_data[1]['name']
    assert 'string payload 1' in stored_data[0]['payload']
    assert 'string payload 2' in stored_data[1]['payload']


def test_log_with_payload():
    event_name = "TestEvent"
    event_payload = {
        "action": "called_api",
        "value": 42
    }

    baserun.log(event_name, payload=event_payload)

    stored_data = baserun_module._thread_local.buffer
    assert stored_data[0]['stepType'] == 'log'
    assert stored_data[0]['name'] == event_name
    assert stored_data[0]['payload'] == event_payload

def test_log_llm_chat():
    config = {
        "model": "gpt-3.5-turbo",
        "temperature": 0.7
    }
    messages = [
        {"role": "system", "content": "You are a helpful assistant."},
        {"role": "user", "content": "Who won the world series in 2020?"}
    ]
    output = "The Los Angeles Dodgers won the World Series in 2020."

    baserun.log_llm_chat(config=config, messages=messages, output=output)

    stored_data = baserun_module._thread_local.buffer
    assert stored_data[0]['stepType'] == 'custom_llm'
    assert stored_data[0]['type'] == "chat"
    assert stored_data[0]['provider'] == "openai"
    assert stored_data[0]['config']['model'] == "gpt-3.5-turbo"
    assert stored_data[0]['messages'] == [
        {"role": "system", "content": "You are a helpful assistant."},
        {"role": "user", "content": "Who won the world series in 2020?"}
    ]
    assert stored_data[0]['output'] == "The Los Angeles Dodgers won the World Series in 2020."


def test_log_llm_completion():
    config = {
        "model": "text-davinci-003",
        "temperature": 0.8,
        "max_tokens": 100
    }
    prompt = "Once upon a time, there was a {character} who {action}."
    variables = {"character": "brave knight", "action": "fought dragons"}

    baserun.log_llm_completion(config=config, prompt=prompt, output="Some random output", variables=variables)

    stored_data = baserun_module._thread_local.buffer
    assert stored_data[0]['stepType'] == 'custom_llm'
    assert stored_data[0]['type'] == "completion"
    assert stored_data[0]['provider'] == "openai"
    assert stored_data[0]['config']['model'] == "text-davinci-003"
    assert stored_data[0]['prompt'] == "Once upon a time, there was a {{character}} who {{action}}."
    assert stored_data[0]['variables'] == {"character": "brave knight", "action": "fought dragons"}
    assert stored_data[0]['output'] == "Some random output"


from enum import Enum, auto


class BaserunProvider(Enum):
    GOOGLE = auto()
    OPENAI = auto()
    LLAMA = auto()


class BaserunType(Enum):
    CHAT = auto()
    COMPLETION = auto()


class BaserunStepType(Enum):
    LOG = auto()
    AUTO_LLM = auto()
    CUSTOM_LLM = auto()


SupportedModels = {
    BaserunProvider.OPENAI: [
        "gpt-3.5-turbo",
        "gpt-3.5-turbo-16k",
        "gpt-3.5-turbo-0613",
        "gpt-3.5-turbo-16k-0613",
        "gpt-4",
        "gpt-4-0613",
        "gpt-4-32k",
        "gpt-4-32k-0613",
        "text-davinci-003",
        "text-curie-001",
        "text-babbage-001",
        "text-ada-001"
    ],
    BaserunProvider.GOOGLE: ["chat-bison@001", "text-bison@001"],
    BaserunProvider.LLAMA: ["llama7b-v2-chat", "llama13b-v2-chat", "llama70b-v2-chat"],
}


def get_provider_for_model(model: str) -> BaserunProvider:
    for provider, models in SupportedModels.items():
        if model in models:
            return provider

    raise ValueError(f"Model '{model}' is not supported by any provider.")



import inspect
import json
import uuid
import os
import requests
import time
import threading
from typing import List, Dict, Union
from urllib.parse import urlparse
import warnings
from .helpers import BaserunProvider, BaserunStepType, BaserunType, get_provider_for_model
from .openai import monkey_patch_openai

_thread_local = threading.local()


class Baserun:
    _initialized = False
    _testExecutions = []
    _api_url = None
    _api_key = None

    @staticmethod
    def init(api_url: str = "https://baserun.ai/api/v1/runs") -> None:
        api_key = os.environ.get('BASERUN_API_KEY')
        if not api_key:
            raise ValueError("Baserun API key is missing. Ensure the BASERUN_API_KEY environment variable is set.")

        if Baserun._initialized:
            warnings.warn("Baserun has already been initialized. Additional calls to init will be ignored.")
            return

        Baserun._api_url = api_url
        Baserun._api_key = api_key
        Baserun._initialized = True

        monkey_patch_openai(Baserun._append_to_buffer)

    @staticmethod
    def test(func):
        if inspect.iscoroutinefunction(func):
            async def wrapper(*args, **kwargs):
                if not Baserun._initialized:
                    return func(*args, **kwargs)

                test_name = func.__name__
                test_inputs = []
                for input_name, input_value in kwargs.items():
                    if inspect.iscoroutine(input_value):
                        input_result = input_value.__name__
                    else:
                        input_result = input_value
                    test_inputs.append(f"{input_name}: {input_result}")

                test_execution_id = str(uuid.uuid4())
                _thread_local.baserun_test_execution_id = test_execution_id
                _thread_local.buffer = []
                start_time = time.time()

                try:
                    result = await func(*args, **kwargs)
                except Exception as e:
                    end_time = time.time()
                    Baserun.store_test({
                        'testName': test_name,
                        'testInputs': test_inputs,
                        'id': test_execution_id,
                        'error': str(e),
                        'startTimestamp': start_time,
                        'completionTimestamp': end_time,
                        "steps": _thread_local.buffer
                    })
                    _thread_local.buffer = []
                    del _thread_local.baserun_test_execution_id
                    raise e

                end_time = time.time()
                Baserun.store_test({
                    'testName': test_name,
                    'testInputs': test_inputs,
                    'id': test_execution_id,
                    'result': str(result) if result is not None else '',
                    'startTimestamp': start_time,
                    'completionTimestamp': end_time,
                    "steps": _thread_local.buffer
                })
                _thread_local.buffer = []
                del _thread_local.baserun_test_execution_id
                return result

        else:
            def wrapper(*args, **kwargs):
                if not Baserun._initialized:
                    return func(*args, **kwargs)

                test_name = func.__name__
                test_inputs = []
                for input_name, input_value in kwargs.items():
                    test_inputs.append(f"{input_name}: {input_value}")

                test_execution_id = str(uuid.uuid4())
                _thread_local.baserun_test_execution_id = test_execution_id
                _thread_local.buffer = []
                start_time = time.time()

                try:
                    result = func(*args, **kwargs)
                except Exception as e:
                    end_time = time.time()
                    Baserun.store_test({
                        'testName': test_name,
                        'testInputs': test_inputs,
                        'id': test_execution_id,
                        'error': str(e),
                        'startTimestamp': start_time,
                        'completionTimestamp': end_time,
                        "steps": _thread_local.buffer
                    })
                    _thread_local.buffer = []
                    del _thread_local.baserun_test_execution_id
                    raise e

                end_time = time.time()
                Baserun.store_test({
                    'testName': test_name,
                    'testInputs': test_inputs,
                    'id': test_execution_id,
                    'result': str(result) if result is not None else '',
                    'startTimestamp': start_time,
                    'completionTimestamp': end_time,
                    "steps": _thread_local.buffer
                })
                _thread_local.buffer = []
                del _thread_local.baserun_test_execution_id
                return result

        return wrapper

    @staticmethod
    def log(name: str, payload: Union[str, Dict]):
        if not Baserun._initialized:
            return

        baserun_test_execution_id = getattr(_thread_local, "baserun_test_execution_id", None)
        if not baserun_test_execution_id:
            warnings.warn("baserun.log was called outside of a Baserun decorated test. The log will be ignored.")
            return

        log_entry = {
            "stepType": BaserunStepType.LOG.name.lower(),
            "name": name,
            "payload": payload,
            "timestamp": time.time(),
        }

        Baserun._append_to_buffer(log_entry)

    @staticmethod
    def log_llm_chat(config: Dict, messages: List[Dict], output: str, variables: Dict[str, str] = None):
        if not Baserun._initialized:
            return

        baserun_test_execution_id = getattr(_thread_local, "baserun_test_execution_id", None)
        if not baserun_test_execution_id:
            warnings.warn(
                "baserun.log_llm_chat was called outside of a Baserun decorated test. The log will be ignored.")
            return

        if 'model' not in config:
            warnings.warn("The 'model' property must be specified in config, falling back to baserun.log")
            Baserun.log(json.dumps({"config": config, "messages": messages, "output": output, "variables": variables}))
            return

        model = config['model']
        provider = get_provider_for_model(model)
        if not isinstance(provider, BaserunProvider):
            warnings.warn(f"The specified model '{model}' is not supported by Baserun, falling back to baserun.log")
            Baserun.log(json.dumps({"config": config, "messages": messages, "output": output, "variables": variables}))
            return

        log_entry = {
            "stepType": BaserunStepType.CUSTOM_LLM.name.lower(),
            "type": BaserunType.CHAT.name.lower(),
            "provider": provider.name.lower(),
            "config": config,
            "messages": [
                {
                    "role": message["role"],
                    "content": message["content"].replace("{", "{{").replace("}", "}}")
                }
                for message in messages
            ],
            "output": output,
            "variables": variables if variables else {},
            "timestamp": time.time(),
        }

        Baserun._append_to_buffer(log_entry)

    @staticmethod
    def log_llm_completion(config: Dict[str, any], prompt: str, output: str, variables: Dict[str, str] = None):
        if not Baserun._initialized:
            return

        baserun_test_execution_id = getattr(_thread_local, "baserun_test_execution_id", None)
        if not baserun_test_execution_id:
            warnings.warn("baserun.log_llm_completion was called outside of a Baserun decorated test. The log will be ignored.")
            return

        if 'model' not in config:
            warnings.warn("The 'model' property must be specified in config, falling back to baserun.log")
            Baserun.log(json.dumps({"config": config, "prompt": prompt, "output": output, "variables": variables}))
            return

        model = config['model']
        provider = get_provider_for_model(model)
        if not isinstance(provider, BaserunProvider):
            warnings.warn(f"The specified model '{model}' is not supported by Baserun, falling back to baserun.log")
            Baserun.log(json.dumps({"config": config, "prompt": prompt, "output": output, "variables": variables}))
            return

        log_entry = {
            "stepType": BaserunStepType.CUSTOM_LLM.name.lower(),
            "type": BaserunType.COMPLETION.name.lower(),
            "provider": provider.name.lower(),
            "config": config,
            "prompt": prompt.replace("{", "{{").replace("}", "}}"),
            "output": output,
            "variables": variables if variables else {},
            "timestamp": time.time(),
        }

        Baserun._append_to_buffer(log_entry)

    @staticmethod
    def store_test(test_data: Dict):
        Baserun._testExecutions.append(test_data)

    @staticmethod
    def flush():
        if not Baserun._initialized:
            warnings.warn("Baserun has not been initialized. No data will be flushed.")
            return

        if not Baserun._testExecutions:
            return

        headers = {"Authorization": f"Bearer {Baserun._api_key}"}
        try:
            response = requests.post(Baserun._api_url, json=json.loads(json.dumps({"testExecutions": Baserun._testExecutions}, default=str)), headers=headers)
            response.raise_for_status()

            response_data = response.json()
            test_run_id = response_data['id']
            parsed_url = urlparse(Baserun._api_url)
            url = f"{parsed_url.scheme}://{parsed_url.netloc}/runs/{test_run_id}"
            return url

        except requests.RequestException as e:
            warnings.warn(f"Failed to upload results to Baserun: {str(e)}")

        Baserun._testExecutions = []

    @staticmethod
    def _append_to_buffer(log_entry: Dict):
        if not hasattr(_thread_local, 'buffer'):
            _thread_local.buffer = []

        _thread_local.buffer.append(log_entry)
